"""Version information for pyrion."""

__version__ = "0.1.20"
__version_info__ = tuple(int(x) for x in __version__.split("."))

# Additional version metadata
__author__ = "Bogdan M. Kirilenko"
__github__ = "https://github.com/kirilenkobm"
__license__ = "MIT"
__copyright__ = "Copyright 2025 Bogdan M. Kirilenko"
